@@ -1,13 +1,17 @@
 #!/bin/sh
 
+set -e
+
 FILTER="grep -v warning"
+CXX=${CXX:=g++}
+CFLAGS_USER=${CFLAGS}
 
 sub()
 {
+CFLAGS="$CFLAGS_USER -Wall -I../ $OPT2"
 
-CFLAGS="-Wall -fno-operator-names -I../ $OPT2"
 echo "compile address.cpp"
-g++ $CFLAGS address.cpp -o address
+$CXX $CFLAGS address.cpp -o address
 
 ./address $1 > a.asm
 echo "asm"
@@ -17,9 +21,9 @@ awk '{printf "%s", sub(/-$/, "", $3) ? $3 : $3 ORS}' a.lst | $FILTER > ok.lst
 echo "xbyak"
 ./address $1 jit > nm.cpp
 echo "compile nm_frame.cpp"
-g++ $CFLAGS -DXBYAK_TEST nm_frame.cpp -o nm_frame
+$CXX $CFLAGS -DXBYAK_TEST nm_frame.cpp -o nm_frame
 ./nm_frame > x.lst
-diff ok.lst x.lst && echo "ok"
+diff -bB ok.lst x.lst && echo "ok"
 
 }
 
